unit jdm84io;                                   { JDM84 IO-routines, v28}
interface                                       { By Jens Dyekjr Madsen}
{$s-}                                           { No stack check        }
const
  err: byte = 0;                                { Returns error value   }
  base: word = 0;                               { Com-port base adress  }
  din: byte = 0;                                { Returns PIC16C84 data }

procedure setcom(n: byte);                      { Select com-port       }
{
  Error Codes:
     1 = Comport not found
     3 = Ilegal parameter value
}

procedure waitdelay(t: word);                   { Delay / ms            }
{ Delay of t ms (minimum, may be longer). About 87s if t=0. Max 5000ms.}

procedure begincmd;                             { Begin command         }
{ Indicates begin of command. Should be called before each command.     }

procedure endcmd;                               { End command           }
{ Indicates end of command. Should be called after each command.        }

procedure mclr;                                 { Reset / go            }
{
  This procedure initializes the communication port and it checks if the
  programmer is found. It does not reset PIC16C84. This is done by dout.
  'Automatic go' function is activated after calling mclr.

  Error Codes:
     1 = Comport not found
     2 = Modem found
     4 = Programmer not found
     5 = Cable too long
     6 = Timer not free
}


{ PIC16C84 routines }

procedure dout(val: byte);                      { In/Out PIC16C84 data  }
{
  Send a bit of the value val to PIC16C84. Recieve data to din.
  Dout resets PIC16C84 and stops 'auto go' first time after mclr.
  Do only call Dout, SDA and SDL between begcmd and endcmd.
}


{ EEPROM (I2C) routines }

procedure sda(val: byte);                       { Set SDA = val         }
procedure scl(val: byte);                       { Set SCL = val         }
function rda: byte;                             { Return SDA            }
{
  SDA sets the value of data. SCL sets the value of clock. 
  Do only call Dout, SDA and SDL between begcmd and endcmd.
  RDA recieves data. Do not use RDA between begcmd and endcmd
  when waiting for acknoledge.
}


implementation

uses delays;                                    { Uses delay routines   }

const
  run: shortint = 0;                            { Run state             }

var
  lastout: byte;                                { Saves last output     }
  i: integer;                                   { Counter etc.          }

procedure setcom(n: byte);                      { Setup com-port        }
begin
  err:=3;                                       { Error 3 if no comport }
  if (n=0) or (n>4) then exit;                  { Check comport value   }
  base:=memw[$40:2*pred(n)];                    { Setup RS232 port      }
  err:=byte(base=0);                            { Return error value    }
  run:=0;                                       { Not run               }
end;

procedure setdlab(d: word);                     { Set Baud Rate Divisor }
var
  lcr: byte;                                    { Saves Line Control Reg}
begin
  lcr:=port[base+3];                            { Store Line Control Reg}
  port[base+3]:=$80 or lcr;                     { Select DLAB           }
  port[base]:=lo(d); port[base+1]:=hi(d);       { Set Baud rate         }
  port[base+3]:=lcr;                            { Restore LCR           }
end;

procedure getdlab(var d: word);                 { Get Baud Rate Divisor }
var
  lcr: byte;                                    { Saves Line Control Reg}
begin
  lcr:=port[base+3];                            { Store Line Control Reg}
  port[base+3]:=$80 or lcr;                     { Select DLAB           }
  d:=port[base]; d:=port[base+1] shl 8 + d;     { Get Baud rate         }
  port[base+3]:=lcr;                            { Restore LCR           }
end;

procedure waitdelay(t: word);                   { Delay / ms            }
begin
  setdlab((longint(t)*1152)div 100+1);          { Calculate baud rate   }
  port[base]:=0;                                { Send data             }
  repeat until port[base+5]>=96;		{ Wait to data send	}
end;

procedure begincmd;                             { Begin command         }
begin
  shortdelay_on;                                { CLI, start timer 2    }
  if run<0 then run:=-run;                      { Run mode active       }
end;

procedure endcmd;                               { End command           }
begin
  shortdelay_off;                               { Close timer 2, STI    }
  if run>0 then run:=-run;                      { Do not run            }
end;

procedure mclr;                                 { Reset / go            }
{ Init comport, check if programmer found, reinitialize if not found.   }
{ Does not reset PIC16C84. Returns with run=1 if ok to run / reset.     }
var
  mstate: array[1..4] of byte;                  { Saves USART state     }
  dlab: word;                                   { Stores Baud Rate Div. }
  j: byte;                                      { Saves port data       }
begin
  run:=0;                                       { Not run               }
  err:=1;                                       { Assume error 1        }
  if base=0 then exit;                          { Exit if comport error }
  err:=6;                                       { Assume error 6        }
  if shortdelay_err then exit;                  { Exit if timer error   }
  err:=2;                                       { Assume error 2        }
  for i:=2 to 4 do mstate[i]:=port[base+i];     { Store USART state     }
  port[base+2]:=6;                              { Disable fifo          }
  port[base+3]:=3;                              { 8 bit, no parity      }
  mstate[1]:=port[base+1];                      { Store IER             }
  port[base+1]:=0;                              { No interrupts         }
  port[base+4]:=0;                              { RTS low, DTR low      }
  getdlab(dlab);                                { Get Baud Rate Divisor }
  waitdelay(1);                                 { Wait 1ms              }
  if port[base+6] and $10 <> 0 then             { Check if modem        }
  begin
    setdlab(dlab);                              { Set Baud Rate Divisor }
    for i:=1 to 4 do port[base+i]:=mstate[i];   { Restore USART state   }
    exit;                                       { and exit              }
  end;
  for i:=0 to 20 do                             { Clock data out, EEPROM}
  begin
    port[base+4]:=succ((i and 1)shl 1);         { DTR high, Toggle RTS  }
    waitdelay(0);                               { Wait 87s             }
  end;
  err:=0;                                       { Assume no error       }
  for i:=1 to 40 do                             { Check if programmer ok}
  begin
    begincmd;                                   { Begin command         }
    port[base+4]:=i and 1;                      { RTS low, Toggle DTR   }
    shortdelay;                                 { Short delay           }
    j:=port[base+6];                            { Read data             }
    endcmd;                                     { End of command        }
    waitdelay(1);                               { Wait 1ms              }
    if ((j shr 4) xor i)and 1<>0 then err:=5;   { Check if error        }
    if ((port[base+6]shr 4)xor i)and 1<>0 then err:=4; { Check if err 4 }
    if err=5 then i:=40;                        { Cable too long error  }
  end;                                          { RTS low, DTR low      }
  if err<>0 then                                { Exit if err<>0        }
  begin
    setdlab(dlab);                              { Set Baud Rate Divisor }
    for i:=1 to 4 do port[base+i]:=mstate[i];   { Restore USART state   }
    exit;                                       { and exit              }
  end;
  waitdelay(1);                                 { Wait 1ms              }
  run:=-1;                                      { Ready to run          }
  lastout:=0;                                   { Update lastout        }
end;


{ PIC16C84 routines }

procedure dout(val: byte);                      { Output PIC16C84 data  }
begin
  if run<=0 then exit;                          { Exit if not run       }
  if run=1 then                                 { Check if need reset   }
  begin                                         { Do PIC16C84 reset     }
    port[base+3]:=$43;                          { Enable Break, no reset}
    endcmd;                                     { Accept interrupts     }
    waitdelay(250);                             { Power on Delay, 250ms }
    setdlab(1);                                 { Setup 115Kbaud,n,8,1  }
    begincmd;                                   { Enable timer 2, cli   }
    port[base+3]:=3;                            { Start reset           }
    for i:=1 to 3 do                            { pulses                }
    begin                                       { to not start auto go  }
      port[base]:=0;                                    { Output zero   }
      repeat until boolean(port[base+5] and $20);       { Wait on empty }
    end;
    port[base+3]:=$43;                          { Enable break, no reset}
    run:=2;                                     { Program mode enabled  }
    waitdelay(0);                               { Delay                 }
  end;
  port[base+4]:=val or 2;                       { Output value, Clk high}
  shortdelay;                                   { Wait on data          }
  din:=(port[base+6] shr 4) and 1;              { Return data           }
  port[base+4]:=val and din;                    { Output value, Clk low }
  shortdelay;                                   { Recharge              }
  if din<>val then for i:=1 to 4 do shortdelay; { Extra rechage delay   }
end;


{ EEPROM (I2C) routines }

function rda: byte;                             { Read data             }
begin
  if run<=0 then rda:=0 else                    { Return zero if not run}
    rda:=(port[base+6] shr 4) and 1;            { Setup rda value       }
end;

procedure sda(val: byte);                       { Set SDA = val         }
begin
  if run<=0 then exit;                          { Exit if not run       }
  lastout:=(lastout and $fe) or val;            { Update lastout        }
  port[base+4]:=lastout;                        { Output lastout        }
  shortdelay;                                   { Short delay           }
end;

procedure scl(val: byte);                       { Set SCL = val         }
begin
  if run<=0 then exit;                          { Exit if not run       }
  lastout:=(lastout and $fd) or 2*val;          { Update lastout        }
  port[base+4]:=lastout;                        { Output lastout        }
  shortdelay;                                   { Short delay           }
end;

end.
